-------------------------------------------------------------------------------
-- $Id$
-------------------------------------------------------------------------------
-- FIFO_Memory.vhd
-------------------------------------------------------------------------------
--
--                  ****************************
--                  ** Copyright Xilinx, Inc. **
--                  ** All rights reserved.   **
--                  ****************************
--
-------------------------------------------------------------------------------
-- Filename:        FIFO_Memory.vhd
--
-- Description:     
--                  
-- VHDL-Standard:   VHDL'93
-------------------------------------------------------------------------------
-- Structure:   
--              FIFO_Memory.vhd
--
-------------------------------------------------------------------------------
-- Author:          goran
-- Revision:        $Revision$
-- Date:            $Date$
--
-- History:
--   goran  2001-06-08    First Version
--
-------------------------------------------------------------------------------
-- Naming Conventions:
--      active low signals:                     "*_n"
--      clock signals:                          "clk", "clk_div#", "clk_#x" 
--      reset signals:                          "rst", "rst_n" 
--      generics:                               "C_*" 
--      user defined types:                     "*_TYPE" 
--      state machine next state:               "*_ns" 
--      state machine current state:            "*_cs" 
--      combinatorial signals:                  "*_com" 
--      pipelined or register delay signals:    "*_d#" 
--      counter signals:                        "*cnt*"
--      clock enable signals:                   "*_ce" 
--      internal version of output port         "*_i"
--      device pins:                            "*_pin" 
--      ports:                                  - Names begin with Uppercase 
--      processes:                              "*_PROCESS" 
--      component instantiations:               "<ENTITY_>I_<#|FUNC>
-------------------------------------------------------------------------------

library ieee;
use ieee.std_logic_1164.all;

entity FIFO_Memory is
  
  generic (
    C_VIRTEXII : boolean;               -- True if target is VII, false if
                                        -- target is Virtex,VirtexE,SpartanII
    C_DEPTH    : natural;               -- Nr of bits needed for the counters
    C_WIDTH    : natural);              -- Nr of bits needed for the data

  port (
    Write_Clk  : in std_logic;
    Write      : in std_logic;
    Write_Addr : in std_logic_vector(C_DEPTH-1 downto 0);
    Write_Data : in std_logic_vector(C_WIDTH-1 downto 0);

    Read_Clk  : in  std_logic;
    Read_Addr : in  std_logic_vector(C_DEPTH-1 downto 0);
    Read_Data : out std_logic_vector(C_WIDTH-1 downto 0)
    );
end entity FIFO_Memory;

library unisim;
use unisim.all;

architecture IMP of FIFO_Memory is

  -----------------------------------------------------------------------------
  -- VirtexII BRAMs
  -----------------------------------------------------------------------------
  component RAMB16_S1_S1
    port (
      DIA   : in  std_logic_vector (0 downto 0);
      DIB   : in  std_logic_vector (0 downto 0);
      ENA   : in  std_ulogic;
      ENB   : in  std_ulogic;
      WEA   : in  std_ulogic;
      WEB   : in  std_ulogic;
      SSRA  : in  std_ulogic;
      SSRB  : in  std_ulogic;
      CLKA  : in  std_ulogic;
      CLKB  : in  std_ulogic;
      ADDRA : in  std_logic_vector (13 downto 0);
      ADDRB : in  std_logic_vector (13 downto 0);
      DOA   : out std_logic_vector (0 downto 0);
      DOB   : out std_logic_vector (0 downto 0) ); 
  end component;

  component RAMB16_S2_S2
    port (
      DIA   : in  std_logic_vector (1 downto 0);
      DIB   : in  std_logic_vector (1 downto 0);
      ENA   : in  std_ulogic;
      ENB   : in  std_ulogic;
      WEA   : in  std_ulogic;
      WEB   : in  std_ulogic;
      SSRA  : in  std_ulogic;
      SSRB  : in  std_ulogic;
      CLKA  : in  std_ulogic;
      CLKB  : in  std_ulogic;
      ADDRA : in  std_logic_vector (12 downto 0);
      ADDRB : in  std_logic_vector (12 downto 0);
      DOA   : out std_logic_vector (1 downto 0);
      DOB   : out std_logic_vector (1 downto 0) ); 
  end component;

  component RAMB16_S4_S4
    port (
      DIA   : in  std_logic_vector (3 downto 0);
      DIB   : in  std_logic_vector (3 downto 0);
      ENA   : in  std_ulogic;
      ENB   : in  std_ulogic;
      WEA   : in  std_ulogic;
      WEB   : in  std_ulogic;
      SSRA  : in  std_ulogic;
      SSRB  : in  std_ulogic;
      CLKA  : in  std_ulogic;
      CLKB  : in  std_ulogic;
      ADDRA : in  std_logic_vector (11 downto 0);
      ADDRB : in  std_logic_vector (11 downto 0);
      DOA   : out std_logic_vector (3 downto 0);
      DOB   : out std_logic_vector (3 downto 0) ); 
  end component;

  component RAMB16_S9_S9
    port (
      DIA   : in  std_logic_vector (7 downto 0);
      DIB   : in  std_logic_vector (7 downto 0);
      DIPA  : in  std_logic_vector (0 downto 0);
      DIPB  : in  std_logic_vector (0 downto 0);
      ENA   : in  std_ulogic;
      ENB   : in  std_ulogic;
      WEA   : in  std_ulogic;
      WEB   : in  std_ulogic;
      SSRA  : in  std_ulogic;
      SSRB  : in  std_ulogic;
      CLKA  : in  std_ulogic;
      CLKB  : in  std_ulogic;
      ADDRA : in  std_logic_vector (10 downto 0);
      ADDRB : in  std_logic_vector (10 downto 0);
      DOA   : out std_logic_vector (7 downto 0);
      DOB   : out std_logic_vector (7 downto 0);
      DOPA  : out std_logic_vector (0 downto 0);
      DOPB  : out std_logic_vector (0 downto 0) ); 
  end component;

  component RAMB16_S18_S18
    port (
      DIA   : in  std_logic_vector (15 downto 0);
      DIB   : in  std_logic_vector (15 downto 0);
      DIPA  : in  std_logic_vector (1 downto 0);
      DIPB  : in  std_logic_vector (1 downto 0);
      ENA   : in  std_ulogic;
      ENB   : in  std_ulogic;
      WEA   : in  std_ulogic;
      WEB   : in  std_ulogic;
      SSRA  : in  std_ulogic;
      SSRB  : in  std_ulogic;
      CLKA  : in  std_ulogic;
      CLKB  : in  std_ulogic;
      ADDRA : in  std_logic_vector (9 downto 0);
      ADDRB : in  std_logic_vector (9 downto 0);
      DOA   : out std_logic_vector (15 downto 0);
      DOB   : out std_logic_vector (15 downto 0);
      DOPA  : out std_logic_vector (1 downto 0);
      DOPB  : out std_logic_vector (1 downto 0) ); 
  end component;

  component RAMB16_S36_S36
    port (
      DIA   : in  std_logic_vector (31 downto 0);
      DIB   : in  std_logic_vector (31 downto 0);
      DIPA  : in  std_logic_vector (3 downto 0);
      DIPB  : in  std_logic_vector (3 downto 0);
      ENA   : in  std_ulogic;
      ENB   : in  std_ulogic;
      WEA   : in  std_ulogic;
      WEB   : in  std_ulogic;
      SSRA  : in  std_ulogic;
      SSRB  : in  std_ulogic;
      CLKA  : in  std_ulogic;
      CLKB  : in  std_ulogic;
      ADDRA : in  std_logic_vector (8 downto 0);
      ADDRB : in  std_logic_vector (8 downto 0);
      DOA   : out std_logic_vector (31 downto 0);
      DOB   : out std_logic_vector (31 downto 0);
      DOPA  : out std_logic_vector (3 downto 0);
      DOPB  : out std_logic_vector (3 downto 0) ); 
  end component;


  -----------------------------------------------------------------------------
  -- Virtex, VirtexE and SpartanII BRAMs
  -----------------------------------------------------------------------------
  component RAMB4_S1_S1
    port (
      DIA   : in  std_logic_vector (0 downto 0);
      DIB   : in  std_logic_vector (0 downto 0);
      ENA   : in  std_ulogic;
      ENB   : in  std_ulogic;
      WEA   : in  std_ulogic;
      WEB   : in  std_ulogic;
      RSTA  : in  std_ulogic;
      RSTB  : in  std_ulogic;
      CLKA  : in  std_ulogic;
      CLKB  : in  std_ulogic;
      ADDRA : in  std_logic_vector (11 downto 0);
      ADDRB : in  std_logic_vector (11 downto 0);
      DOA   : out std_logic_vector (0 downto 0);
      DOB   : out std_logic_vector (0 downto 0)); 
  end component;

  component RAMB4_S2_S2
    port (
      DIA   : in  std_logic_vector (1 downto 0);
      DIB   : in  std_logic_vector (1 downto 0);
      ENA   : in  std_ulogic;
      ENB   : in  std_ulogic;
      WEA   : in  std_ulogic;
      WEB   : in  std_ulogic;
      RSTA  : in  std_ulogic;
      RSTB  : in  std_ulogic;
      CLKA  : in  std_ulogic;
      CLKB  : in  std_ulogic;
      ADDRA : in  std_logic_vector (10 downto 0);
      ADDRB : in  std_logic_vector (10 downto 0);
      DOA   : out std_logic_vector (1 downto 0);
      DOB   : out std_logic_vector (1 downto 0)); 
  end component;

  component RAMB4_S4_S4
    port (
      DIA   : in  std_logic_vector (3 downto 0);
      DIB   : in  std_logic_vector (3 downto 0);
      ENA   : in  std_ulogic;
      ENB   : in  std_ulogic;
      WEA   : in  std_ulogic;
      WEB   : in  std_ulogic;
      RSTA  : in  std_ulogic;
      RSTB  : in  std_ulogic;
      CLKA  : in  std_ulogic;
      CLKB  : in  std_ulogic;
      ADDRA : in  std_logic_vector (9 downto 0);
      ADDRB : in  std_logic_vector (9 downto 0);
      DOA   : out std_logic_vector (3 downto 0);
      DOB   : out std_logic_vector (3 downto 0)); 
  end component;

  component RAMB4_S8_S8
    port (
      DIA   : in  std_logic_vector (7 downto 0);
      DIB   : in  std_logic_vector (7 downto 0);
      ENA   : in  std_ulogic;
      ENB   : in  std_ulogic;
      WEA   : in  std_ulogic;
      WEB   : in  std_ulogic;
      RSTA  : in  std_ulogic;
      RSTB  : in  std_ulogic;
      CLKA  : in  std_ulogic;
      CLKB  : in  std_ulogic;
      ADDRA : in  std_logic_vector (8 downto 0);
      ADDRB : in  std_logic_vector (8 downto 0);
      DOA   : out std_logic_vector (7 downto 0);
      DOB   : out std_logic_vector (7 downto 0)); 
  end component;

  component RAMB4_S1_S1
    port (
      DIA   : in  std_logic_vector (15 downto 0);
      DIB   : in  std_logic_vector (15 downto 0);
      ENA   : in  std_ulogic;
      ENB   : in  std_ulogic;
      WEA   : in  std_ulogic;
      WEB   : in  std_ulogic;
      RSTA  : in  std_ulogic;
      RSTB  : in  std_ulogic;
      CLKA  : in  std_ulogic;
      CLKB  : in  std_ulogic;
      ADDRA : in  std_logic_vector (7 downto 0);
      ADDRB : in  std_logic_vector (7 downto 0);
      DOA   : out std_logic_vector (15 downto 0);
      DOB   : out std_logic_vector (15 downto 0)); 
  end component;

  
  type VII_BRAMS_TYPE is (B16_S1_S1, B16_S2_S2, B16_S4_S4,
                          B16_S9_S9, B16_S18_S18, B16_S36_S36);


  signal null_16 : std_logic_vector(15 downto 0);
  signal null_2  : std_logic_vector(1 downto 0);
  
begin  -- architecture IMP

  null_16 <= (others => '0');
  null_2  <= (others => '0');
  
  Using_VII : if (C_VIRTEXII) generate
    RAMB16_S18_S18_I : RAMB16_S18_S18
      port map (
        DIA   => Write_Data,            -- [in  std_logic_vector (15 downto 0)]
        DIB   => null_16,               -- [in  std_logic_vector (15 downto 0)]
        DIPA  => null_2,                -- [in  std_logic_vector (1 downto 0)]
        DIPB  => null_2,                -- [in  std_logic_vector (1 downto 0)]
        ENA   => '1',                   -- [in  std_ulogic]
        ENB   => '1',                   -- [in  std_ulogic]
        WEA   => Write,                 -- [in  std_ulogic]
        WEB   => '0',                   -- [in  std_ulogic]
        SSRA  => '0',                   -- [in  std_ulogic]
        SSRB  => '0',                   -- [in  std_ulogic]
        CLKA  => Write_Clk,             -- [in  std_ulogic]
        CLKB  => Read_Clk,              -- [in  std_ulogic]
        ADDRA => Write_Addr,            -- [in  std_logic_vector (9 downto 0)]
        ADDRB => Read_Addr,             -- [in  std_logic_vector (9 downto 0)]
        DOA   => open,                  -- [out std_logic_vector (15 downto 0)]
        DOB   => Read_Data,             -- [out std_logic_vector (15 downto 0)]
        DOPA  => open,                  -- [out std_logic_vector (1 downto 0)]
        DOPB  => open);                 -- [out std_logic_vector (1 downto 0)]
  end generate Using_VII;


end architecture IMP;

