-------------------------------------------------------------------------------
-- $Id$
-------------------------------------------------------------------------------
-- lut_ram.vhd
-------------------------------------------------------------------------------
--
--                  ****************************
--                  ** Copyright Xilinx, Inc. **
--                  ** All rights reserved.   **
--                  ****************************
--
-------------------------------------------------------------------------------
-- Filename:        lut_ram.vhd
--
-- Description:     
--                  
-- VHDL-Standard:   VHDL'93
-------------------------------------------------------------------------------
-- Structure:   
--              lut_ram.vhd
--
-------------------------------------------------------------------------------
-- Author:          goran
-- Revision:        $Revision$
-- Date:            $Date$
--
-- History:
--   goran  2002-01-21    First Version
--
-------------------------------------------------------------------------------
-- Naming Conventions:
--      active low signals:                     "*_n"
--      clock signals:                          "clk", "clk_div#", "clk_#x" 
--      reset signals:                          "rst", "rst_n" 
--      generics:                               "C_*" 
--      user defined types:                     "*_TYPE" 
--      state machine next state:               "*_ns" 
--      state machine current state:            "*_cs" 
--      combinatorial signals:                  "*_com" 
--      pipelined or register delay signals:    "*_d#" 
--      counter signals:                        "*cnt*"
--      clock enable signals:                   "*_ce" 
--      internal version of output port         "*_i"
--      device pins:                            "*_pin" 
--      ports:                                  - Names begin with Uppercase 
--      processes:                              "*_PROCESS" 
--      component instantiations:               "<ENTITY_>I_<#|FUNC>
-------------------------------------------------------------------------------

library IEEE;
use IEEE.std_logic_1164.all;

library unisim;
use unisim.all;

entity LUT_RAM is
  generic (
    INIT_S : string
    );
  port (
    -- Processor Interface
    Clk         : in  std_logic;
    Addr        : in  std_logic_vector(3 downto 0);
    Write       : in  std_logic;
    Write_Data  : in  std_logic;
    Read_Data   : out std_logic;
    Addr2       : in  std_logic_vector(3 downto 0);
    Read_Data2  : out std_logic
    );

end entity LUT_RAM;

architecture IMP of LUT_RAM is

  component RAM16x1D is
    -- pragma translate_off
    generic (
      XOn : boolean
      );
    -- pragma translate_on
    port (
      WE    : in  std_logic;
      D     : in  std_logic;
      WClk  : in  std_logic;
      A0    : in  std_logic;
      A1    : in  std_logic;
      A2    : in  std_logic;
      A3    : in  std_logic;
      DPRA0 : in  std_logic;
      DPRA1 : in  std_logic;
      DPRA2 : in  std_logic;
      DPRA3 : in  std_logic;
      SPO   : out std_logic;
      DPO   : out std_logic);
  end component RAM16x1D;

  attribute INIT : string;

  attribute INIT of LUT_RAM_I : label is INIT_S;
  
begin  -- architecture IMP

  LUT_RAM_I : RAM16x1D
      -- pragma translate_off
      generic map (
        XOn => False)                     -- [boolean]
      -- pragma translate_on
      port map (
        WE    => Write,                 -- [in  std_logic]
        D     => Write_Data,            -- [in  std_logic]
        WClk  => Clk,                   -- [in  std_logic]
        A0    => Addr(0),               -- [in  std_logic]
        A1    => Addr(1),               -- [in  std_logic]
        A2    => Addr(2),               -- [in  std_logic]
        A3    => Addr(3),               -- [in  std_logic]
        DPRA0 => Addr2(0),              -- [in  std_logic]
        DPRA1 => Addr2(1),              -- [in  std_logic]
        DPRA2 => Addr2(2),              -- [in  std_logic]
        DPRA3 => Addr2(3),              -- [in  std_logic]
        SPO   => Read_Data,             -- [out std_logic]
        DPO   => Read_Data2);           -- [out std_logic]

end architecture IMP;
